#include "include.h"
#include "sbuf.h"
#include "protocol2/blist.h"

struct slist *slist_alloc(void)
{
	struct slist *slist=NULL;
	if(!(slist=(struct slist *)calloc_w(1, sizeof(struct slist), __func__))
	  || !(slist->blist=blist_alloc()))
		slist_free(&slist);
	return slist;
}

void slist_free(struct slist **slist)
{
	struct sbuf *sb;
	struct sbuf *shead;
	if(!slist || !*slist) return;
	blist_free(&(*slist)->blist);
	shead=(*slist)->head;
	while(shead)
	{
		sb=shead;
		shead=shead->next;
		sbuf_free(&sb);
	}
	free_v((void **)slist);
}

void slist_add_sbuf(struct slist *slist, struct sbuf *sb)
{
	if(slist->tail)
	{
		// Add to the end of the list.
		slist->tail->next=sb;
		slist->tail=sb;
		// Markers might have fallen off the end. Start them again
		// on the tail.
		if(!slist->last_requested) slist->last_requested=slist->tail;
		if(!slist->add_sigs_here) slist->add_sigs_here=slist->tail;
		if(!slist->blks_to_request) slist->blks_to_request=slist->tail;
		if(!slist->blks_to_send) slist->blks_to_send=slist->tail;
	}
	else
	{
		// Start the list.
		slist->head=sb;
		slist->tail=sb;
		// Pointers to the head that can move along the list
		// at a different rate.
		slist->last_requested=sb;
		slist->add_sigs_here=sb;
		slist->blks_to_request=sb;
		slist->blks_to_send=sb;
	}
}
